#include <avr/io.h>
#include <avr/interrupt.h>
#define F_CPU 16000000
#include <util/delay.h>
#define AX 3
#define AY 3
#define LEFT \
	!(PINA & (1 << PA6))

#define RIGHT \
	!(PINA & (1 << PA7))

#define UP \
	!(PINA & (1 << PA0))

#define DOWN \
	!(PINA & (1 << PA5))
	
#define NORMALIZE(speed) \
	if (speed >= 256) \
		speed = 256; \
	else if (speed <= -256) \
		speed = -256



void CommandBoat(int* speedX, int* speedY)
{
	// sa eliminam +/- 1, ca erorile se pot propaga
	// si barca poate deveni instabila
	if (*speedX <= 1 && *speedX >= -1)
	{
		*speedX = 0;
	}
	if (*speedY <= 1 && *speedY >= -1)
	{
		*speedY = 0;
	}
	
	// nu merg cu spatele
	if (*speedY <= 0)
		*speedY = 0;
		
	int _speedX = *speedX;
	int _speedY = *speedY;

	OCR1A = (_speedY - _speedX < 0) ? 0 : (_speedY - _speedX);
	OCR2A = (_speedX + _speedY >= 256) ? 255 : (_speedX + _speedY);
	
}		
		

int main()
{
	int speedX = 0;
	int speedY = 0;
	// Pe aici intra cele patru input-uri de la antena
	// inainte
	DDRA &= ~(1 << PA0);
	PORTA |= (1 << PA0);
	// inapoi
	DDRA &= ~(1 << PA5);
	PORTA |= (1 << PA5);
	// stanga
	DDRA &= ~(1 << PA6);
	PORTA |= (1 << PA6);
	// dreapta
	DDRA &= ~(1 << PA7);
	PORTA |= (1 << PA7);
	// INPUT 1 si 2
	DDRA |= (1 << PA1);
	DDRA |= (1 << PA2);
	// PC1 = 1, PC2 = 0 - mergi inainte
	PORTA |= (1 << PA1);
	PORTA &= ~(1 << PA2);

	// INPUT 3 si 4
	DDRA |= (1 << PA3);
	DDRA |= (1 << PA4);
	// PA3 = 1, PA4 = 0 - mergi inainte
	PORTA |= (1 << PA3);
	PORTA &= ~(1 << PA4);
	
	//motor 1  - timer 1 fast PWM
    // PD5 output - OC1A este PD5
	 DDRD |= (1 << PD5);
	 //Conform tabelului 16-5 din datasheet, pentru modul Fast PWM 8-bit, bi?ii WGMn0 si WGMn2 au valoarea 1
	 TCCR1A |= (1 << WGM10); 
	 //TCCR1A con?ine doar bi?ii WGM10 si WGM11, WGM12 ?i WGM13 se gasesc in TCCR1B
	 //Conform tabelului 16-3 din datasheet, pentru modul non-inverting COM1A1 = 1 ?i COM1A0 = 0
	 TCCR1A |= (1 << COM1A1);
	 //Conform tabelului 16-6 din datasheet, pentru Prescaler de 1024 scriem 1 pe CS12 si CS10
	 TCCR1B |= (1 << CS12) | (1 << CS10);
	 //Pragul la care se schimba semnalul (Fig. 4); fill-rate 0.5: jumatate din timp 1, jumatate din timp 0
	 //Deoarece n acest mod TOP este 0xFF, OCR1A reprezinta efectiv frecven?a
	 OCR1A = 0;
	 
	 
	 // motor 2 - se comanda prin fast PWM pe timer 2
	 //PD5 output - OC2A este PD7
	 DDRD |= (1 << PD7);
	 //Conform tabelului 16-5 din datasheet, pentru modul Fast PWM 8-bit, bi?ii WGMn0 si WGMn2 au valoarea 1
	 TCCR2A |= (1 << WGM20); 
	 //TCCR1A con?ine doar bi?ii WGM10 si WGM11, WGM12 ?i WGM13 se gasesc in TCCR1B
	 TCCR2A |= (1 << WGM21);
	 //Conform tabelului 16-3 din datasheet, pentru modul non-inverting COM1A1 = 1 ?i COM1A0 = 0
	 TCCR2A |= (1 << COM2A1);
	 //Conform tabelului 16-6 din datasheet, pentru Prescaler de 1024 scriem 1 pe CS12 si CS10
	 TCCR2B |= (1 << CS22) | (1 << CS21) | (1 << CS20);
	 //Pragul la care se schimba semnalul (Fig. 4); fill-rate 0.5: jumatate din timp 1, jumatate din timp 0
	 //Deoarece n acest mod TOP este 0xFF, OCR1A reprezinta efectiv frecven?a
	 OCR2A = 0;
 
	 while (1)
	 {
		if (LEFT)
		{
			speedX -= AX;
		}
		else
		{
			if (!RIGHT)
				speedX = 0;
		}
		if (RIGHT)
		{
			speedX += AX;
		}
		else
		{
			if (!LEFT)
				speedX = 0;
		}
		NORMALIZE(speedX);
		
		
		if (UP)
		{
			speedY += AY;
		}
		else
		{
			if (!DOWN)
				speedY -= (AY / 3);
		}
		if (DOWN)
		{
			speedY -= AY;
		}
		else
		{
			if (!UP)
				speedY += (AX / 3);
		}
		NORMALIZE(speedY);
		
		CommandBoat(&speedX, &speedY);
		_delay_ms(100);
	 }
	 
}